import string
from django.utils.translation import ugettext_lazy as _

"""
" Default django settings for Token Asena
" NOTE: Remember to append _DEFAULT.
"""

ASENA_CHAR_SET_DEFAULT=str(string.ascii_letters + string.digits)

""" For security you can allow or forbid certain messages to be shown.
"""
ASENA_SECURITY_CONFIG_DEFAULT = {
    'show_timeout_error' : True,
    'show_expired_error' : True,
    'show_disabled_error' : True,
    'show_invalid_error' : True,
}

ASENA_ERROR_MESSAGES_DEFAULT = {
    'general' : _("Sorry; you are not authorized to view this page."),
    'timeout' : _("Your session has expired."),
    'expired' : _("The token is no longer valid."),
    'disabled' : _("That token has been disabled."),
    'invalid' : _("The token is not valid."),
}

"""
" The name for the session cookie.
"""
ASENA_SESSION_NAME_DEFAULT='asena_token_value'

"""
" The name of the session timeout cookie.
"""
ASENA_SESSION_TIMEOUT_NAME_DEFAULT='asena_token_timeout'

"""
" The date format for converting to and from a date.
" :see: datetime.datetime.strptime
"""
ASENA_DATETIME_FORMAT_DEFAULT="%c"

"""
" The name of the GET or POST key.
"""
ASENA_URL_KEY_DEFAULT='token'

"""
" A regexp for how to split a "session timeout." For example, if the
" ASENA_TIMEOUT_SEPARATORS value is r'[hms]', the value '0h4m2s' will be
" parsed as datetime.timedelta(0, 4, 2)
"""
ASENA_TIMEOUT_SEPARATORS_DEFAULT=r'[\,\:]'



from django.shortcuts import render, redirect
from django.conf import settings
from django.core.exceptions import PermissionDenied
#from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponse, JsonResponse
from django.views.decorators.http import require_POST, require_GET

import string, pprint

from asena.models import Token, TokenException, InvalidTokenException
from asena.models import AuthorizationException, DisabledTokenException
from asena.forms import TokenWall
from asena.utils import *

from django.conf import settings

logger = logging.getLogger('to_terminal')

@require_GET
def token_ajax_generate(request, *args, **kwargs):
    """ Generate token text (not an actual token, though) and return the
    text as an ajax response.
    
    TODO: Make the default length of "10" a setting.
    """
    
    length = int(kwargs.pop('length', 10))
    charset = string.ascii_letters + string.digits
    
    response = JsonResponse(random_chars(charset, length))
    
    if get_setting('ALLOW_CORS_FOR_TESTING', False):
        response['Access-Control-Allow-Origin' : '*']
    

def token_set_ajax_generate(request, *args, **kwargs):
    
    logger.debug("kwargs: %s"%(pprint.pformat(kwargs)))
    
    count = int(kwargs.pop('count', -1))
    length = int(kwargs.pop('length', -1))
    charset = string.ascii_letters + string.digits
    
    if count < 1:
        count = 1
    if length < 1:
        length = 1
        
    token_set = [random_chars(charset, length) for i in range(0, count)]
    #token_set = [i for i in range(0, count)]
        
    return HttpResponse(','.join(token_set))
    

def token_wall(request, *args, **kwargs):
    context = {}
    
    token_form = TokenWall()
    
    template = get_setting('ASENA_TOKEN_WALL_TEMPLATE', 'token_wall.html')
    
    context.update({'token_wall_form' : token_form})
    
    if request.method == 'GET' and 'token' in request.GET:
        context.update({'token' : request.GET['token']})
    elif request.method == 'POST' and 'token' in request.POST:
        context.update({'token' : request.POST['token']})
        
    return render(request, template, context)

"""
" Yes, I know I need to use `reverse` to get the URL, but if the asena URLs are 
" included in a project's URLs, Django complains that the URLs are empty 
" because of a circular import. This constant gets around that.
"""
TOKEN_WALL_URL='/token_wall/'

def token_protect(redirect_url=TOKEN_WALL_URL):
    """ Only allow the user to access the page if a token is given.
    
    :param redirect_url: (optional). Redirect the view to ``redirect``. Default
        is '/tokenwall?next=<url>'
    :type redirect: def
    
    :return: A view function; ``decorate.call`` if the token given is valid or 
        ``redirect`` if the token is invalid.
    """
    
    logger.debug("In token_protect('%s')..."%redirect_url)
    
    def wrap(view_func, _redirect_url=redirect_url):
        
        def _wrapped_view_func(request, _redirect_url=_redirect_url, *args, 
            **kwargs):
                
            session_key = get_default_setting('ASENA_SESSION_NAME')
            url_key = get_default_setting('ASENA_URL_KEY')
            session_time_key = get_default_setting('ASENA_SESSION_TIMEOUT_NAME')
           
            if has_session_expired(request.session):
                for k in (session_key, session_time_key):
                    if k in request.session:
                        del request.session[k]
            else:
                remain = get_session_time_remaining(request.session)
                remain = remain.total_seconds()
                logger.info("Session time remaining: %d seconds."%remain)
                return view_func(request, *args, **kwargs)

            # Here we'll check to see if the token is valid. If an exception is
            # thrown, return a redirect.

            logger.debug("Token wall request method was %s"%request.method)
            if request.method == 'POST':
                #logger.debug("POST data: %s"%pprint.pformat(request.POST.dict()))
                token_wall_form = TokenWall(data=request.POST.dict())
            elif request.method == 'GET':
                #logger.debug("GET data: %s"%pprint.pformat(request.GET.dict()))
                token_wall_form = TokenWall(data=request.GET.dict())
            else:
                #logger.error("Something happened!")
                raise PermissionDenied('Expected either GET or POST.')
            
            token = token_wall_form.get_token()
                
            if token:
                logger.debug("Token %s checks out!"%token)
                request.session.update(token.get_session())
            else:
                logger.warn("Could not verify token. Returning " +
                        "token_wall view.")
                kwargs.update({'next' : _redirect_url})
                return token_wall(request, *args, **kwargs)

            
            # If everything checks out, return the view function.
            return view_func(request, *args, **kwargs)
        return _wrapped_view_func
    return wrap



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from asena.models import Token, TokenSet
from asena.widgets import TokenWidget
from asena.fields import TokenField, TokenSetField, TimeDeltaField

from asena.utils import (random_chars, random_chars_set, get_setting, 
    get_default_setting)

import logging, pprint
logger = logging.getLogger('to_terminal')

class TokenWall(forms.Form):
    token = forms.CharField(label="Token Text")
   
    def clean(self):
        cleaned_data = super(TokenWall, self).clean()

        logger.debug("Cleaning data: %s"%pprint.pformat(cleaned_data))
        
        token_value = None
        if 'token' in cleaned_data:
            token_value = cleaned_data['token']
        
        # Validate the data.
        asena_security = get_default_setting('ASENA_SECURITY_CONFIG')
        asena_errors = get_default_setting('ASENA_ERROR_MESSAGES')
        
        code = None
        
        t = None
        if token_value:
            t = Token.objects.get(value=token_value)
            
        if ((not t) or t.has_expired() or t.is_disabled()):
            code = 'general'
        
        if (not t) and asena_security['show_invalid_error']:
            code = 'invalid'
        
        elif t.has_expired() and asena_security['show_timeout_error']:
            code = 'expired'
                
        elif t.is_disabled() and asena_security['show_disabled_error']:
            code='disabled'
        
        if code:
            raise forms.ValidationError(asena_errors[code], code)
        
        logger.debug("Cleaned data: %s"%pprint.pformat(cleaned_data))

    def get_token(self):
        """ If the form is valid, return the token from the token value.
        """
        if not self.is_valid():
            logger.warn("TokenWall form data is not valid.")
            return None
    
        tt = self.cleaned_data['token']
        logger.debug("Looking for token '%s'"%tt)
        return Token.objects.get(value=tt)
        
    
class TokenCreationForm(forms.ModelForm):
    value = TokenField()
    comment = forms.CharField(widget=forms.Textarea)
    
    def clean(self):
        cleaned_data = super(TokenCreationForm, self).clean()
        return cleaned_data
    
    class Meta:
        model = Token
        fields = ['value', 'comment']

class TokenSetCreationForm(forms.ModelForm):
    count = forms.IntegerField(label="Number of tokens", required=False,
        help_text="Leave blank to update a token set.")
    length = forms.IntegerField(label="Length of each token", required=False,
        help_text="Leave blank to udpate a token set.")
    session_timeout = TimeDeltaField(required=False,
            label="For how long will the session be valid?",
            help_text="Note that the session will automatically end " +
                "when the browser data is cleared.")
    
    def __init__(self, *args, **kwargs):
        logger.debug("Token Set:")
        logger.debug("args\t%s"%(pprint.pformat(args)))
        logger.debug("kwargs\t%s"%(pprint.pformat(kwargs)))
        super(TokenSetCreationForm, self).__init__(*args, **kwargs)
        #self.instance = kwargs.pop('instance', None)
        
    def clean(self, *args, **kwargs):
        cleaned_data = super(TokenSetCreationForm, self).clean(*args, **kwargs)
        self.cleaned_data = cleaned_data
        return cleaned_data
    
    def save(self, force_insert=False, force_update=False, commit=True):
        t = super(TokenSetCreationForm, self).save(commit=False)
        
        count = self.cleaned_data.pop('count', 0) or 0
        length = self.cleaned_data.pop('length', 0) or 0
        
        values = []
        
        logger.debug("Timeout: %s"%self.cleaned_data.get('session_timeout',
            None))
        
        if self.instance:
            ts = self.instance.get_tokens()
            count = count - len(ts)
            if count < 1: count = 0
            token_set = super(TokenSetCreationForm, self).save(commit=True)
            
        else:
            token_set = TokenSet.objects.create(**self.cleaned_data)
        
        if count > 0 and length > 0:
            values = random_chars_set(get_default_setting('ASENA_CHAR_SET'), 
                length, count)
            logger.debug("Generate tokens: %s"%pprint.pformat(values))
            
        for v in values:
            Token.objects.create(value=v, token_set=token_set)
        
        return token_set
    
    class Meta:
        model = TokenSet
        fields = ['name', 'disabled', 'session_timeout', 'expiration',
                  'count', 'length', ]



from django import forms
from asena.widgets import *

from django.conf import settings
import logging, pprint, os
logger = logging.getLogger('to_terminal')

class TokenField(forms.MultiValueField):
    #widget = TokenWidget
    
    def __init__(self, *args, **kwargs):
        token_value = kwargs.pop('token_value', None)
        self.widget = TokenWidget(attrs={'token_value' : token_value})
        fields = (forms.CharField(), forms.IntegerField())
        super(TokenField, self).__init__(fields, *args, **kwargs)
    
    def compress(self, data_list):
        logger.debug("Compressing data list %s"%pprint.pformat(data_list))
        if len(data_list) >= 1:
            return data_list[0]
        return None
    
class TokenSetField(forms.MultiValueField):
    def __init__(self, *args, **kwargs):
        self.widget = TokenSetWidget()
        fields = (
            forms.MultipleChoiceField(),
        )
        super(TokenSetField, self).__init__(fields, *args, **kwargs)
        
    def compress(self, data_list):
        logger.debug("Compressing data list %s"%pprint.pformat(data_list))
        if len(data_list) > 0:
            return data_list.join(',')
        return ''

class TimeDeltaField(forms.MultiValueField):
        
    fields = (
        forms.IntegerField(label="Hours", min_value=0, max_value=100),
        forms.IntegerField(label="Minutes", min_value=0, max_value=60),
    )
    
    widget=TimeDeltaWidget
    
    def clean(self, value):
        #v = super(TimeDeltaField, self).clean(value)
        #logger.debug("Cleaning %s -> %s"%(value, v))
        return self.compress(value)

    def compress(self, data_list):
        """ Compress the TimeDeltaField. If any fields are left blank,
        fill them in with a zero.
        """
        dl = data_list + [0,]*(2 - len(data_list or []))
        data = ','.join([str(d or '0') for d in dl])
        logger.debug("Compressing %s -> %s."%(data_list, data))
        return data



from django.db import models
from django.utils.translation import ugettext_lazy as _
from asena.utils import random_chars, get_default_setting
import string, random
from datetime import timedelta, datetime

import logging, pprint
logger = logging.getLogger('to_terminal')

"""
" Token Exceptions
"""

class TokenException(Exception):
    """ A general abstract exception related to tokens.
    "
    """
    def __init__(self, token=None):
        """
        " :param token: the invalid token passed in
        " :type token: object (typically a Token or string)
        """
        self.token = token
        
    def __repr__(self):
        return "%s: %s"%(self.__class__.__name__, self.token)
    
class InvalidTokenException(TokenException):
    """ Thrown when a token is invalid (not in the database)
    """
    def __init__(self, *args, **kwargs):
        super(AutorizationException, self).__init__()
    
class DisabledTokenException(TokenException):
    """ Thrown when a token is valid but has been disabled.
    """
    def __init__(self, *args, **kwargs):
        super(AutorizationException, self).__init__()
    
class AuthorizationException(TokenException):
    """ Thrown when a user has tried to access a page for which they need a 
        token.
    """
    def __init__(self, *args, **kwargs):
        super(AuthorizationException, self).__init__()

class TokenSet(models.Model):
    generated = models.DateTimeField(auto_now=True)
    name = models.CharField(blank=True, null=True, max_length=200)
    disabled = models.BooleanField(default=False)
    expiration = models.DateTimeField(blank=True, null=True)
    session_timeout = models.CharField(blank=True, null=True, max_length=20,
        help_text=_("Time Delta format (e.g. 'HH,MM,SS')"))
    
    @classmethod
    def generate_set(Klass, count, length=10, name=None):
        """ Convenience method to generate a set of tokens.
        
         :param count: Number of tokens to generate.
         :type count: int
         
         :param name: An optional name to use for the token set.
         :type name: str
         
         :return: A new TokenSet with the newly-generated tokens.
        """
        token_set = Klass.objects.create(name=name)
        for i in range(0, count):
            Token.generate(length=length, token_set=token_set, comment=name)
        return token_set
    
    def has_expired(self):
        return ( self.expiration ) and (datetime.now() > self.expiration)
    
    def get_tokens(self):
        return Token.objects.filter(token_set__pk=self.pk)
            
    def __unicode__(self):
        ts = self.get_tokens().count()
        if self.name:
            return "%s (%d tokens)"%(self.name, ts)
        return "%d tokens generated %s"%(ts, self.generated)

    class Meta:
        abstract = False
        app_label = "asena"
        verbose_name = "Token Set"
        verbose_name_plural = "Token Sets"


class Token(models.Model):
    """ A randomly-generated string.
    
    :param value: The value of the string (generated by machine).
    :type value: str
    
    :param comment: A comment for the token (optional).
    :type comment: str
    
    :type token_set: A foreign key to TokenSet.
    :type token_set: models.ForeignKey
    
    """
    value = models.CharField(max_length=200)
    comment = models.TextField(max_length=1000, null=True, blank=True)
    token_set = models.ForeignKey("TokenSet", related_name="tokens",
                                  related_query_name="tokens")
    disabled = models.BooleanField(default=False)
    expiration = models.DateTimeField(blank=True, null=True)
    session_timeout = models.CharField(blank=True, null=True, max_length=20,
        help_text=_("Note that the token will expire when session" +
            " cookies are cleared."))
    
    _REQUEST_KEY='token'
    
    @classmethod
    def generate(Klass, length, token_set, comment=None,
                 valid_characters=str(string.letters + string.digits)):
        """ Generate a random token.
        
        :param length: The length of the token.
        :type length: int
        
        :param token_set: The assoicated token_set.
        :type token_set: TokenSet
        
        :param comment: A comment for the token (optional).
        :type comment: str
        
        :param valid_characters: A string of valid characters to use for the
            token. Optional. Default is string.letters + string.digits.
        :type valid_characters: str
        
        :return: The token just generated.
        """
        global random_chars
        val = random_chars(valid_characters, length)
        token = Klass.objects.create(token_set=token_set, comment=comment,
                                    value=val)
        return token
    
    def is_disabled(self):
        return (self.token_set.has_expired() or 
            self.token_set.disabled)
    
    def has_expired(self):
        return self.get_expiration() and (self.get_expiration() > 
            datetime.now())
            
    """
    " Methods for the session
    """
    
    def get_session(self):
        """ Get the dict values required for a session variable.
        """
        session_name_key = get_default_setting('ASENA_SESSION_NAME')
        session_time_key = get_default_setting('ASENA_SESSION_TIMEOUT_NAME')
        datetime_format = get_default_setting('ASENA_DATETIME_FORMAT')

        exp = self.get_session_expiration().strftime(datetime_format)
        
        return {
            session_name_key : self.value,
            session_time_key : exp,
        }
    
    def get_session_timeout(self, as_timedelta=False):
        """ Get the current session timeout. If it's null (or "0,0"), then
        return the TokenSet's session timeout.
        
        :param as_timedelta: True if we should conver to timedelta.
        :type as_timedelta: bool
        
        :return: The session timeout value.
        """
        val = self.token_set.session_timeout
        if (self.session_timeout) or (self.session_timeout != '0,0'):
            val = self.token_set.session_timeout
        if as_timedelta and val:
            (hours, minutes) = val.split(',')
            return timedelta(hours=int(hours), minutes=int(minutes))
        return val
    
    def get_session_expiration(self, init=datetime.now()):
        """ Get the timeout for the token. This will be calculated from the
        initial time ``init``
        
        :param init: The initial datetime. Default is datetime.datetime.now()
        :type init: datetime.
        """
        timeout = self.get_session_timeout()
        if timeout:
            return init + self.get_session_timeout(True)
        return None
        
    
    def get_expiration(self):
        """ Get either the token's expiration or (if the token set has an
            expiration) the token set's expiration.
            
            :return: The token expiration.
        """
        if self.token_set.expiration:
            return self.token_set.expiration
        return self.expiration
    
    @classmethod
    def exists(self, value):
        logger.debug("Checking if token value '%s' exists..."%value)
        return Token.objects.filter(value=value).exists()
    
    @classmethod
    def is_valid(self, value):
        if Token.exists(value):
            t = Token.objects.get(value=value)
            return not t.is_disabled
        return False

    def has_session_expired(self, expiration_time):
        """
        " True if the session has expired.
        """

    @classmethod
    def request_is_valid(Klass, request):
        """ Given a request, check that the payload contains a token and that
        the token is valid. If it's invalid, throw an exception.
        
        :return: True if a token is valid for a given request. Throws an 
                exception otherwise.
        """
        if request.method == 'POST':
            arr = request.POST
        elif request.method == 'GET':
            arr = request.GET
        else:
            raise AuthorizationException()
        
        if Klass._REQUEST_KEY not in arr:
            raise AuthorizationException()
        
        tok = arr[Klass._REQUEST_KEY]
        qs = Klass.objects.filter(value=tok)
        
        if not qs.exists():
            raise InvalidTokenException(tok)
        
        if qs[0].is_disabled():
            raise DisabledTokenException(qs[0])
        
        return True
    
    def __unicode__(self):
        if self.comment:
            return "%s - %s"%(self.comment, self.value)
        return "%s - %s's token set"%(self.value, self.token_set.name)

    class Meta:
        abstract = False
        app_label = "asena"
        verbose_name = "Token"
        verbose_name_plural = "Tokens"



import random
from datetime import datetime, timedelta

from django.conf import settings

from asena import settings_default

import logging
logger = logging.getLogger('to_terminal')
import pprint

def random_chars(char_set, length):
    """ Choose random characters from a set.
    
    :param char_set: The character set.
    :type char_set: str
    
    :param length: the length of the set.
    :type length: int
    
    :return: A string of random characters chosen from char_set.
    """
    s = ""
    sz = len(char_set)-1
    r = random.SystemRandom()
    for i in range(0, length):
        n = r.randint(0, sz)
        s = s + char_set[n]
    return s

def random_chars_set(char_set, length, count):
    out_set = []
    for i in range(0, count):
        out_set.append(random_chars(char_set, length))
    return out_set

def html_attrs(attrs):
    """ Convert the attrs in dict format to an HTML format.
    
    :param attrs: The attributes for the element
    :type attrs: dict
    
    :return: string representing the attributes portion of a tag.
    """
    html = ""
    for a in attrs.items():
        if a[1]:
            html = html + "%s=\"%s\" "%(a)
    return html

def get_setting(setting, alt_value):
    if hasattr(settings, setting):
        return settings.setting
    return alt_value

def get_default_setting(setting):
    default_setting_key='%s_DEFAULT'%setting
    if hasattr(settings, setting):
        return settings.setting
    elif hasattr(settings_default, default_setting_key):
        return getattr(settings_default, default_setting_key)
    else:
        raise AttributeError("Missing default value ''%s``"%default_setting_key)

def make_url(base, **kwargs):
    sep = '?'
    logger.debug("Making URL with %s and %s"%(base, pprint.pformat(kwargs)))
    for k,v in kwargs.items():
        base = str(base + sep + k + '=' + v)
        if sep == '?':
            sep = '&'
    return base

def get_session_time_remaining(session_data, _date=datetime.now()):
    """ Calculate the amount of time remaining for the session as a
    datetime.timedelta object. 

    :param session_data: The client's session.
    :type session_data: backends.base.SessionBase

    :param _date: The initial date to test. Default is datetime.datetime.now()
                  Mainly used for testing.
    :default _date: datetime.datetime.now()
    :type _date: datetime.datetime

    :return: The amount of time remaining as a timedelta object.

    """
    timeout_name = get_default_setting('ASENA_SESSION_TIMEOUT_NAME')
    session_name = get_default_setting('ASENA_SESSION_NAME')
    dt_format = get_default_setting('ASENA_DATETIME_FORMAT')

    # Assume that if the session data does not exist, it has expired.
    if not session_data:
        return None

    exp_date = session_data.get(timeout_name, None)

    if not exp_date:
        return None

    exp_date = datetime.strptime(exp_date, dt_format)

    logger.debug("It is now %s. The session wil end at %s"%(
        _date, exp_date))

    diff = exp_date - _date

    # We can think of timedelta() as the int value ``0``. This tests if there
    # is a negative difference in time.
    if diff <= timedelta():
        return timedelta()

    # Otherwise, just return the difference.
    return diff

def has_session_expired(session_data, _date=datetime.now()):
    """ True if the token session has expired.
       :return: True if the session has expired, False otherwise.
    """
    remain = get_session_time_remaining(session_data, _date)
    if remain:
        return not (remain > timedelta())
    return True



from django.contrib import admin
from asena.models import *
from asena.forms import *

import logging
logger = logging.getLogger('to_terminal')

class TokenAdmin(admin.ModelAdmin):
    form = TokenCreationForm

class TokenSetAdmin(admin.ModelAdmin):
    form = TokenSetCreationForm

admin.site.register(TokenSet, TokenSetAdmin)
#admin.site.register(TokenSet)
logger.debug("Registered TokenSet")
admin.site.register(Token, TokenAdmin)
#admin.site.register(Token)
logger.debug("Registered Token")


from django.conf.urls import *  # NOQA
from django.conf.urls.i18n import i18n_patterns

import asena.views

urlpatterns = patterns('',
    url(r'^token_wall/$', asena.views.token_wall, name="token_wall"),
    url(r'^token/generate/(?P<count>\d+)/(?P<length>\d+)/$',
        asena.views.token_set_ajax_generate,
        name="generate_token_set"),
    #url(r'^token/generate/(?P<length>\d+)/$', asena.views.token_ajax_generate, 
        #name="generate_token"),
)


#!/usr/bin/env python

from distutils.core import setup

####### Used for compatability with python 2.2.3 ######
from sys import version
if version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None
########################################################

with open('README.rst') as file:
    long_description = file.read()
    file.close()

setup(
    # Required information
    name='django-token-asena',
    version='1.0b',         # major.minor[.patch[.sub]]
    url='https://www.gitorious.org/django-token-asena',
    
    # Switch out
    author='Jordan Hewitt',          # or user maintainer
    author_email='jordannh@sent.com',    # or use maintainer_email
    maintainer='',
    maintainer_email='',
    
    # optional
    description = "Generate tokens and token sets for temporary session " +
        "authentication",
    long_description=long_description,
    download_url='',
    classifiers=['Topic :: Internet :: WWW/HTTP :: WSGI :: Application', ],
    platforms=['', ''],
    license='GPLv3',
    
    #data_files = ['./docs',],
    packages = ['asena', ],
)



import re

from django import forms
from django.utils.text import mark_safe
from django.utils.translation import ugettext_lazy as _
from django.templatetags.static import static
from django.conf import settings

from asena.utils import html_attrs

import logging, pprint, os
logger = logging.getLogger('to_terminal')

""" Constant to pass in to the "onClick" HTML attribute for a button
"""
ONCLICK_BUTTON_METHOD="generateAsenaTokens('%(id)s'); return false;"

""" In case the static URL changes, make it a constant, too
"""
STATIC_JS='tokenGeneration.js'

class Button(forms.Widget):
    def __init__(self, attrs):
        self.attrs = attrs
        self.label = self.attrs.pop('label', '')
        self.on_click_method = attrs.pop('onClick', None)
        
    def render(self, name, value, attrs=None):
        """ Render the button.
        
        If given an ``onClick`` attribute, id of the widget will be passed in to
        the function parameters.
        
        :param name: The name of the widget
        :type name: str
        
        :param value: The value of the widget (unused).
        :type value: void
        
        :param attrs: Attributes for the widget.
        :type attrs: dict
        """
        if 'id' in attrs and self.on_click_method:
            attrs.update({
                'onClick' : str(self.on_click_method%{'id' : attrs['id'],}),
                'type' : 'button',
            })
            
        # It's ugly, but first we need to prepend some javascript.
        js = static(STATIC_JS)
        
        s = """<script type="text/javascript"></script>
<script type="text/javascript" src="%s"></script>"""%js
        
        return mark_safe(s + "<button %s>%s</button>"%(html_attrs(attrs),
                                                   self.label))
    
class SizeWidget(forms.NumberInput):
    def render(self, name, value, attrs):
        html = super(SizeWidget, self).render(name, value, attrs)
        return mark_safe('<label for="%s">Size:</label>%s'%(name, html))

class TokenWidget(forms.MultiWidget):
    
    def __init__(self, attrs={}):
        """ A widget to generate a token.
        
        :param attrs:   attributes for the widget. So far attrs is largely
                        ignored except for the key ``token_value`` wich is the 
                        value of the token we want to generate.
        
        :type attrs:    dict
        """
        logger.debug("Token attributes: %s"%pprint.pformat(attrs))
        token_value = attrs.pop('token_value', None)
        text_attrs = {'disabled' : '1',}
        if token_value:
            text_attrs.update({'value' : token_value,})
        widgets = [forms.TextInput(text_attrs),
                   SizeWidget(),
                   Button(attrs={'onClick' : ONCLICK_BUTTON_METHOD,}),
               ]
        super(TokenWidget, self).__init__(widgets, attrs)
    
    def decompress(self, value):
        if not value:
            return [None,]
        logger.debug("Decompressing value \"%s\""%value)
        return [value,]
    
class TokenSetWidget(forms.MultiWidget):
    def __init__(self, attrs={}):
        button_attrs = {
            'label' : 'Generate',
            'onClick' : ONCLICK_BUTTON_METHOD,
        }
        widgets = (
            forms.SelectMultiple(),
            Button(button_attrs),
        )
        super(TokenSetWidget, self).__init__(widgets, attrs)
    
    def decompress(self, value):
        if not value:
            return [None,]
        logger.debug("Decompressing value \"%s\""%value)
        return [value,]

class TimeDeltaWidget(forms.MultiWidget):
    
    def __init__(self, attrs=None):
        
        default_style = 'width: 75px; float: left;'
        default_min = 0
        
        hours = forms.NumberInput({
            'style' : default_style,
            'min' : default_min,
            'max' : 100,
            'label' : 'Hours',
        })
        
        minutes = forms.NumberInput({
            'style' : default_style,
            'min' : default_min,
            'max' : 60,
            'label' : 'Minutes',
        })
        
        widgets = (hours, minutes)
        
        super(TimeDeltaWidget, self).__init__(widgets=widgets, attrs=attrs)
        
    def _get_attr_from_rendered_widget(self, attr, rendered_widget):
        """ Get an attribute from a rendered widget.
        
        :param attr: the attribute to extract.
        :type attr: string
        
        :param rendered_widget: The widget that's already been rendered.
        :type attr: unicode
        """
        attr_re = r'.+%(attr)s\=[\'"](?P<%(attr)s>[^"\']+)[\'"].+'%(
            {'attr' : attr})
        matches = re.match(attr_re, rendered_widget)
        if matches:
            return matches.group(attr)
        return None
    
    def _get_name_from_rendered_widget(self, rendered_widget):
        """ Get the ``name`` attribute value.
        
        :param rendered_widget: The widget that's already been rendered.
        :type attr: unicode
        
        :return: the name attribute value.
        """
        return self._get_attr_from_rendered_widget('name', rendered_widget)
    
    def _get_label_from_rendered_widget(self, rendered_widget):
        """ Get the ``label`` attribute value.
        
        :param rendered_widget: The widget that's already been rendered.
        :type attr: unicode
        
        :return: the label attribute value.
        """
        return self._get_attr_from_rendered_widget('label', rendered_widget)
        
    def format_output(self, rendered_widgets):
        """ Return the HTML for this multiwidget
        
        :see: forms.MultiWidget.format_output
        """
        s = u''
        
        label_style="padding: 5px 5px; margin: 0px 0px; " + \
            "display: inline-block; " + \
            "width: auto;";
        
        """ Here's what's tricky: we already have 'label' and 'name' attributes
        stored in the widget. But we're passed already-renderd widgets. So,
        for every widget we need to extract out the 'label' and 'name' 
        attributes from the rendered widgets. Ugly hack, but it works.
        """
        for i in range(0, len(rendered_widgets)):
            rw = rendered_widgets[i]
            name = self._get_name_from_rendered_widget(rw)
            label_text = self._get_label_from_rendered_widget(rw) or ''
            s = s + u'<span style="display:inline-block;">' + \
                unicode(rendered_widgets[i]) + \
                u'<label style="%s" '%label_style + \
                u'for="%s">%s</label>'%(name, label_text) + \
                u'</span>'
            
        #return u''.join(rendered_widgets)
        return s

    def decompress(self, value):
        val2 = [None]*2
        if value:
            val2 = [int(v) for v in value.split(',')]
        logger.debug("Decompressing %s -> %s"%(value, pprint.pformat(val2)))
        return val2



import logging, pprint
logger = logging.getLogger('to_terminal')

#import sys, os
#from django.db import models
#BASE_DIR=os.path.dirname(os.path.abspath(__file__))
#sys.path.insert(0, BASE_DIR)
#logger.debug("Inserted %s. Now sys.path is: %s"%(BASE_DIR,
            #pprint.pformat(sys.path)))
    
#from . import (
    #admin,
    #fields,
    #forms,
    #models,
    #urls,
    #utils,
    #views,
    #widgets,
#)